--[[
Plugin for range check
]]

----------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("RangeCheck")

----------------------------------
-- Locals
local LSM = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0-Plugins")
local Party

local db


-- Credits to Kollektiv and DXE letting me use the map dimensions from DXE
-- http://www.wowace.com/addons/deus-vox-encounters/
-------------------------
-- MAP DIMENSIONS
-------------------------

local MapDims= {
	Ulduar = {
		[1] = {w = 3064.9614761023, h = 2039.5413309668}, 	-- Expedition Base Camp
		[2] = {w = 624.19069622949, h = 415.89374357805}, 	-- Antechamber of Ulduar
		[3] = {w = 1238.37427179,   h = 823.90183235628}, 	-- Conservatory of Life
		[4] = {w = 848.38069183829, h = 564.6688835337}, 	-- Prison of Yogg-Saron
		[5] = {w = 1460.4694647684, h = 974.65312886234},  -- Spark of Imagination
		[6] = {w = 576.71549337896, h = 384.46653291368},  -- The Mind's Eye (Under Yogg)
	},
	Naxxramas = {
		[1] = {w = 1018.3655494957, h = 679.40523953718}, -- Construct
		[2] = {w = 1019.1310739251, h = 679.18864376555}, -- Arachnid
		[3] = {w = 1118.1083638787, h = 744.57895516418}, -- Military
		[4] = {w = 1117.0809918236, h = 745.97398439776}, -- Plague
		[5] = {w = 1927.3190541014, h = 1284.6530841959}, -- Entrance
		[6] = {w = 610.62737087301, h = 407.3875157986},  -- KT/Sapphiron
	},
	TheObsidianSanctum = {
		[0] = {w = 1081.6334214432, h = 721.79860069158},
	},
	TheEyeofEternity = {
		[1] = {w = 400.728405332355, h = 267.09113174487},
	},
	TheArgentColiseum = {
		[1] = {w = 344.20785972537, h = 229.57961178118},
		[2] = {w = 688.60679691348, h = 458.95801567569},
	},
	VaultofArchavon = {
		[1] = {w = 842.2254908359, h = 561.59878021123},
	},
	IcecrownCitadel = {
		[1] = {w = 1262.8025621533, h = 841.91669450207}, -- The Lower Citadel
		[2] = {w = 993.25701607873, h = 662.58829476644}, -- The Rampart of Skulls
		[3] = {w = 181.83564716405, h = 121.29684810833}, -- Deathbringer's Rise
		[4] = {w = 720.60965618252, h = 481.1621506613}, -- The Frost Queen's Lair
		[5] = {w = 1069.6156745738, h = 713.83371679543}, -- The Upper Reaches
		[6] = {w = 348.05218433541, h = 232.05964286208}, -- Royal Quarters
		-- [7] = {w = 0, h = 0},
	},
	Dalaran = {
		[1] = {w = 829.97011173593, h = 553.36899735997}
	}
}


local defaults = {
	profile = {
		UpdateFreq = 0.2,
		PluginEnabled = true,
		Width = 180,
		range = 15,
		maxNames = 5,
		GrowUp = true,
		x = 0,
		y = 0,
	}
}

for i, setting in pairs(RW.defaults.profile) do
	defaults.profile[i] = setting
end

local inset = defaults.profile.BorderSize > 2 and defaults.profile.Insets or 0
local backdrop = {
	tile = false,
	insets = { left = inset, right = inset, top = inset, bottom = inset },
}

--------------------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("RangeCheck", defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
end

function plugin:PluginEnable()
	if not db.PluginEnabled then
		self:Disable()
		return
	end
	
	self:RegisterMessage("RW_ConfigMode")
	self:RegisterMessage("RW_DefaultsChanged")
	self:RegisterMessage("RW_ShowRangeCheck")
	Party = RW:GetPlugin("Party")
end

function plugin:PluginDisable()
end

local function dragStart(self)
	self:StartMoving()
end
local function dragStop(self)
	self:StopMovingOrSizing()
	local scale = self:GetEffectiveScale() / UIParent:GetEffectiveScale()
	
	local x, y = self:GetCenter()
	x, y = x * scale, y * scale
	
	x = x - GetScreenWidth()/2
	y = y - GetScreenHeight()/2
	
	db.x = x
	db.y = y
	
	plugin:SendMessage("RW_GUIValuesChanged")
end

------------------------------------
-- Anchor

local function getAnchor()
	if plugin.anchor then return plugin.anchor end
	local frame = RW:CreateAnchor(plugin:GetName(), UIParent, "Range Check")
	local s = frame:GetEffectiveScale()
	frame:SetScript("OnDragStart", dragStart)
	frame:SetScript("OnDragStop", dragStop)
	
	plugin.anchor = frame
	
	plugin:Reset()
	return frame
end

function plugin:Reset()
	db = self.db.profile
	if plugin.anchor then
		local frame = plugin.anchor
		frame:ClearAllPoints()
		local scale = frame:GetEffectiveScale() / UIParent:GetEffectiveScale()
		frame:ClearAllPoints()
		frame:SetPoint("CENTER", UIParent, "CENTER", db.x / scale, db.y / scale)		
		frame:SetWidth(db.Width)
		
		self:RW_DefaultsChanged("RW_DefaultsChanged", "none")
	end
end

local function onUpdate(self, elapsed)
	self.t = (self.t or 0) + elapsed
	if self.t > db.UpdateFreq then
		local numFound = 0
		local partyType = Party:GetPartyType()
		for i = 1, Party:GetPartySize() do
			local uID = partyType..i
			if (not UnitIsUnit("player", uID)) and not UnitIsDeadOrGhost(uID) then
				local name, range = plugin:CheckRange(uID)
				if name then
					numFound = numFound + 1
					self.names[numFound]:SetText(UnitName(uID).." ("..range..")")
					local color = RAID_CLASS_COLORS[select(2, UnitClass(uID))]
					self.names[numFound]:SetTextColor(color.r, color.g, color.b)
					self.names[numFound]:Show()
				end
			end
			if numFound >= plugin.db.profile.maxNames then break end
		end
		for i = numFound + 1, #self.names do
			self.names[i]:SetText()
			self.names[i]:Hide()
		end
		self:SetHeight(8 + self.rangeString:GetStringHeight() + 6 + numFound * (self.names[1]:GetStringHeight() + 6))
		self.t = 0
	end
end

local function getMainWindow()
	if plugin.mainWindow then return plugin.mainWindow end
	local anchor = getAnchor()
	
	local frame = CreateFrame("Frame", "RWRFrame", UIParent)
	frame:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT")
	frame:SetPoint("BOTTOMRIGHT", anchor, "TOPRIGHT")
	frame:EnableMouse(true)
	frame:RegisterForDrag("LEFTBUTTON")
	frame:SetMovable(true)
	frame:SetHeight(30)
	
	local rangeString = frame:CreateFontString("RWR_TitleString", "ARTWORK")
	rangeString:SetPoint("TOPLEFT", frame, "TOPLEFT", 6, -6)
	frame.rangeString = rangeString
	
	--Close button
	local closebutton = CreateFrame("Button", "RWR_CloseButton", frame)
	closebutton:SetWidth(14)
	closebutton:SetHeight(14)
	closebutton:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -4, -6)
	
	local t = closebutton:CreateTexture()
	t:SetAllPoints(closebutton)
	t:SetTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
	t:SetTexCoord(0.2, 0.8, 0.2, 0.8)
	closebutton:SetNormalTexture(t)
	
	t = closebutton:CreateTexture(nil, "BACKGROUND")
	t:SetTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
	t:SetAllPoints(closebutton)
	t:SetTexCoord(0.2, 0.8, 0.2, 0.8)
	closebutton:SetPushedTexture(t)
	
	t = closebutton:CreateTexture()
	t:SetTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	t:SetAllPoints(closebutton)
	t:SetBlendMode("ADD")
	t:SetTexCoord(0.2, 0.8, 0.2, 0.8)
	closebutton:SetHighlightTexture(t)
	
	local buttonUp = CreateFrame("Button", "RWR_UpButton", frame)
	buttonUp:SetWidth(16)
	buttonUp:SetHeight(16)
	buttonUp:SetPoint("TOPRIGHT", closebutton, "TOPLEFT", -4, 0)
	
	local t = buttonUp:CreateTexture()
	t:SetAllPoints(buttonUp)
	t:SetTexture("Interface\\Buttons\\UI-PlusButton-Up")
	buttonUp:SetNormalTexture(t)
	
	t = buttonUp:CreateTexture(nil, "BACKGROUND")
	t:SetTexture("Interface\\Buttons\\UI-PlusButton-Down")
	t:SetAllPoints(buttonUp)
	buttonUp:SetPushedTexture(t)
	
	t = buttonUp:CreateTexture()
	t:SetTexture("Interface\\Buttons\\UI-PlusButton-Hilight")
	t:SetAllPoints(buttonUp)
	t:SetBlendMode("ADD")
	buttonUp:SetHighlightTexture(t)
	
	local buttonDown = CreateFrame("Button", "RWR_UpButton", frame)
	buttonDown:SetWidth(16)
	buttonDown:SetHeight(16)
	buttonDown:SetPoint("TOPRIGHT", buttonUp, "TOPLEFT", -2, 0)
	
	local t = buttonDown:CreateTexture()
	t:SetAllPoints(buttonDown)
	t:SetTexture("Interface\\Buttons\\UI-MinusButton-Up")
	buttonDown:SetNormalTexture(t)
	
	t = buttonDown:CreateTexture(nil, "BACKGROUND")
	t:SetTexture("Interface\\Buttons\\UI-MinusButton-Down")
	t:SetAllPoints(buttonDown)
	buttonDown:SetPushedTexture(t)
	
	t = buttonDown:CreateTexture()
	t:SetTexture("Interface\\Buttons\\UI-PlusButton-Hilight")
	t:SetAllPoints(buttonDown)
	t:SetBlendMode("ADD")
	buttonDown:SetHighlightTexture(t)
	
	local names = {}
	for i = 1, db.maxNames do
		names[i] = frame:CreateFontString(frame:GetName().."_Name_"..i, "ARTWORK")
		names[i]:SetPoint("TOPLEFT", i == 1 and frame.rangeString or names[i-1], "BOTTOMLEFT", i == 1 and 6 or 0, -6)
		names[i]:Hide()
	end
	
	frame.closebutton = closebutton
	frame.buttonUp = buttonUp
	frame.buttonDown = buttonDown
	frame.names = names
	plugin.mainWindow = frame
	
	plugin:ApplySettings()
	
	frame:SetScript("OnUpdate", onUpdate)
	closebutton:SetScript("OnClick", function(self, button)
		if button == "LeftButton" then
			self:GetParent():Hide()
		end
	end)
	buttonUp:SetScript("OnClick", function(self, button)
		if button == "LeftButton" then
			plugin:IncreaseRange()
		end
	end)
	buttonDown:SetScript("OnClick", function(self, button)
		if button == "LeftButton" then
			plugin:DecreaseRange()
		end
	end)
	
	return frame
end

function plugin:ApplySettings()
	if not self.mainWindow then return end
	local frame = getMainWindow()
	local rangeString = frame.rangeString
	local anchor = getAnchor()
	
	frame:SetWidth(db.Width)
	
	backdrop.bgFile = LSM:Fetch("background", db.BGTexture)
	backdrop.edgeFile = LSM:Fetch("border", db.Border)
	backdrop.edgeSize = db.BorderSize
	frame:SetBackdrop(backdrop)
	frame:SetBackdropColor(unpack(db.BGColor))
	frame:SetBackdropBorderColor(unpack(db.BorderColor))
	
	rangeString:SetFont(LSM:Fetch("font", db.Font), db.FontSize, db.FontOutline)
	rangeString:SetTextColor(unpack(db.FontColor))
	rangeString:SetText(L.RANGE:format(db.range))
	
	for i, name in pairs(frame.names) do
		name:SetFont(LSM:Fetch("font", db.Font), db.FontSize, db.FontOutline)
	end
	
	frame:ClearAllPoints()
	if db.GrowUp then
		frame:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT")
	else
		frame:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT")
	end
end

function plugin:IncreaseRange()
	db.range = db.range + 1
	getMainWindow().rangeString:SetText(L.RANGE:format(db.range))
end

function plugin:DecreaseRange()
	db.range = db.range - 1
	getMainWindow().rangeString:SetText(L.RANGE:format(db.range))
end

function plugin:CheckRange(uID)	
	local range = RW:GetDistanceToUnit(uID)
	
	if not range then return end
	
	return range < db.range, ("%.1f"):format(range)
end

--- Show range check
-- @param show Indicates if the window should be shown or hidde (true/false)
function plugin:RW_ShowRangeCheck(message, show, range)
	if range then
		db.range = range
		getMainWindow().rangeString:SetText(L.RANGE:format(db.range))
	end
	if show then
		getMainWindow():Show()
	elseif not show and self.anchor then
		getMainWindow():Hide()
	end
end

function plugin:RW_DefaultsChanged(message, option, value)
	if db[option] then db[option] = value end
	self:ApplySettings()
end

function plugin:RW_ConfigMode(message, start)
	if start then
		getAnchor():Show()
		self:RW_ShowRangeCheck("RW_ShowRangeCheck", true)
	else
		getAnchor():Hide()
		self:RW_ShowRangeCheck("RW_ShowRangeCheck", false)
	end
end

------------------------------------
-- Options

function plugin:GetOptions()
	return "Plugins", "Alert Plugins", "Range Check", self.GetGUI, [[Interface\Icons\INV_Misc_Map_01]]
end

local function changeCallback()
	plugin:Reset()
end

function plugin:GetGUI()
	local self = self or plugin
	local UIF = LibStub("AceGUI-Factory-1.0"):GetFactory("RW")
	local AceGUI = LibStub("AceGUI-3.0")
	
	local group, label, slider, check, igroup, button, color, select
	
	group = AceGUI:Create("ScrollFrame")
	group:SetLayout("Flow")
	
	label = UIF:MainTitleLabel("Range Check")
	group:AddChild(label)
	
	igroup = UIF:InlineGroup("")
	group:AddChild(igroup)	
	
	label = UIF:NormalText(L.RANGE_CHECK_DESC)
	label:SetImage([[Interface\Icons\INV_Misc_Map_01]])
	igroup:AddChild(label)
	
	igroup = UIF:InlineGroup("")
	group:AddChild(igroup)	
	
	check = UIF:CheckBox("Enabled", db, "PluginEnabled", function() 
		if not db.PluginEnabled then
			self:Disable()
		else
			self:Enable()
		end
	end)
	igroup:AddChild(check)
	
	slider = UIF:Slider("Update Frequency", db, "UpdateFreq", 0.05, 2, 0.05)
	igroup:AddChild(slider)
	
	label = UIF:NormalText(" ")
	label:SetFullWidth(true)
	igroup:AddChild(label)
	
	check = UIF:CheckBox("Grow Up", db, "GrowUp", changeCallback)
	check:SetDescription(L.GROW_UP_DESC)
	check:SetFullWidth(false)
	igroup:AddChild(check)
	
	slider = UIF:Slider("Width", db, "Width", 100, 300, 1, changeCallback)
	igroup:AddChild(slider)
	
	local sizeGroup = UIF:InlineGroup2("Position")
	igroup:AddChild(sizeGroup)	
	
	slider = UIF:Slider("X", db, "x", 
		-floor(GetScreenWidth() * UIParent:GetEffectiveScale()), 
		floor(GetScreenWidth() * UIParent:GetEffectiveScale()),
		1, changeCallback)
	slider:SetRelativeWidth(0.5)
	sizeGroup:AddChild(slider)
	
	slider = UIF:Slider("Y", db, "y", 
		-floor(GetScreenHeight() * UIParent:GetEffectiveScale()), 
		floor(GetScreenHeight() * UIParent:GetEffectiveScale()),
		1, changeCallback)
	slider:SetRelativeWidth(0.5)
	sizeGroup:AddChild(slider)
	
	button = UIF:Button("Show", function()
		plugin:SendMessage("RW_ShowRangeCheck", true)
	end)
	igroup:AddChild(button)
	
	button = UIF:Button("Hide", function()
		plugin:SendMessage("RW_ShowRangeCheck", false)
	end)
	igroup:AddChild(button)
	
	button = UIF:Button(L.RESET_SETTINGS, function() 
		plugin.db:ResetProfile() 
	end)
	button:SetUserData("confirm", true)
	button:SetUserData("TriggerGlobalUpdate", true)
	group:AddChild(button)
	
	group:FixScroll()
	
	return group
end

do
	local started, startX, startY, facing
	SLASH_RWSIZECALC1 = "/msize"
	SlashCmdList["RWSIZECALC"] = function()
		if started then
			local x, y = GetPlayerMapPosition("player")
			local width = math.sin(math.pi + facing) * 20 / (x - startX)
			local height = math.cos(math.pi + facing) * 20 / (y - startY)
			print(("%s = { [%d] = {w = %s, h = %s}}"):format(GetMapInfo(), GetCurrentMapDungeonLevel(), width, height))
			started = false
		else
			WorldMapFrame:Show()
			WorldMapFrame:Hide()
			startX, startY = GetPlayerMapPosition("player")
			facing = GetPlayerFacing()
			started = true
		end
	end
end
